#include "first.h"
#include <fstream>
#include <iostream>
#include <algorithm>
#include "io.h"
#include "main.h"
#include "varParaMem.h"
#include "distribution.h"

// If defined, would have too many ddi pairs
// #define _INACTIVE_MOTPAIR_IN_SS

// g_curjProbChange 0: g_curjProb matches the current state. 1: otherwise
// ??? in current version it is only not updated after every state change for efficiency
// reason. Only need to make sure, it's updated before every call to compute jProb.
// NUMITERCHECK: # iterations to do before checking if to stop
int g_numProt, g_numMot, g_curjProbChange = 1, NUMITERCHECK = 3;
// g_aPos, g_aNeg: determines the prior for the motif affinity.
// See usage info
double g_curjProb, g_aPos, g_aN2P = 1.0, g_aNeg, g_aAll, IMPROVETHRESH = 0.01;

void ReadOption(int & argc, char ** & argv, int & leakType, int & perType, ifstream & ddiFile, ifstream & ddmdlFile, ifstream & ddioptFile, int & pertSize)
{
  char opt;
  // Remove the executable name
  argv++;
  argc--;
  if (argc == 0) {
    PrintUsage(cerr);
    exit(1);
  }
  while (argv[0][0] == '-') {
    opt = argv[0][1];
    argv++;
    argc--;
    switch (opt) {
    case 'l':
      leakType = atoi(argv[0]);
      argv++;
      argc--;
      break;
    case 'p':
      perType = atoi(argv[0]);
      argv++;
      argc--;
      break;
    case 'd':
      Mylib::InFopen(ddiFile, argv[0]);
      argv++;
      argc--;
      Mylib::InFopen(ddmdlFile, argv[0]);
      argv++;
      argc--;
      Mylib::InFopen(ddioptFile, argv[0]);
      argv++;
      argc--;
      break;
    case 'n':
      g_aN2P = atof(argv[0]);
      argv++;
      argc--;
      break;
    case 's':
      pertSize = atoi(argv[0]);
      argv++;
      argc--;
      break;
    case 'f':
      g_numProt = atoi(argv[0]);
      argv++;
      argc--;
      g_numMot = atoi(argv[0]);
      argv++;
      argc--;
      break;
    default:
      PrintUsage(cerr);
      exit(1);
    }
  }
}

int main(int argc, char * argv[])
{
  // perType:
  //   1: force a motif and a protein to be non-binding. Our model
  //   2: force pairs between motif types to be non-binding. Riley's model
  //   3: force a pair of motifs in a protein pair to be non-binding
  // Default: 1
  int i, leakType, perType, pertSize = -1;
  double jprob;
  ifstream protFile, modelFile, pmFile, motFile, protLenFile, motCoverFile, predpairFile, predbindFile, ddiFile("/dev/null"), ddmdlFile("/dev/null"), ddioptFile("/dev/null");
  ofstream outFile, predictFile;
  string cmdline;

  cmdline = "*** Command line:";
  for (i = 0; i < argc; i++) {
    cmdline += ((string)(" ") + argv[i]);
  }
  cmdline += "\n";

  leakType = 1;
  perType = 1;
  ReadOption(argc, argv, leakType, perType, ddiFile, ddmdlFile, ddioptFile, pertSize);
  if (argc != 14) {
    PrintUsage(cerr);
    exit(1);
  }

  // Never change: pi
  // Update in EnM1(): mip, leakProb, and om (accumulating sufficient statistics).
  // Don't change: mip.GetActPair().
  // In PertPair(): the nonbinding info in pm is updated
  ProtMot pm(g_numProt, g_numMot);
  MotInterPot mip(g_numMot);
  ProtInter pi(g_numProt), di(g_numProt);
  ObsModel om, dm;
  pm.SetLeakType(leakType);

  Mylib::InFopen(protFile, argv[0]);
  Mylib::InFopen(modelFile, argv[1]);
  Mylib::InFopen(pmFile, argv[2]);
  Mylib::InFopen(motFile, argv[3]);
  Mylib::OutFopen(outFile, argv[6]);
  ofstream nullFile("/dev/null");
  Mylib::InFopen(protLenFile, argv[11]);
  Mylib::InFopen(motCoverFile, argv[13]);
  outFile.precision(15);
  g_aPos = atof(argv[10]);
  g_aNeg = g_aPos * g_aN2P;
  g_aAll = g_aPos + g_aNeg;

  outFile << cmdline;
  outFile << "*** # proteins:\t" << g_numProt << endl;
  outFile << "*** # motifs:\t" << g_numMot << endl;
  outFile << "Prior:\t" << g_aPos << '\t' << g_aN2P << '\t' << g_aNeg << '\t' << g_aAll << endl;
  outFile << "pertSize:\t" << pertSize << endl;

  ReadFiles(pm, pi, mip, om, dm, di, protLenFile, motCoverFile, pmFile, protFile, modelFile, motFile, outFile, atoi(argv[12]), ddiFile, ddmdlFile, ddioptFile);
  pm.Print(outFile);
  outFile << "****** PPI ******" << endl;
  pi.Print(pm, outFile);
#ifndef _INACTIVE_MOTPAIR_IN_SS
  outFile << "****** DDI ******" << endl;
  di.Print(pm, outFile);
#endif // _INACTIVE_MOTPAIR_IN_SS
  outFile << "****** Potential without conditioning ******" << endl;
  mip.Print(outFile);
  outFile << "****** PPI observatoin model ******" << endl;
  om.Print(outFile);
  outFile << "****** DDI observatoin model ******" << endl;
  dm.Print(outFile);
  outFile << "***********************************************" << endl;
  outFile << "*** Initialization done." << endl;
  outFile << "***********************************************" << endl;
  MotInterPot micp(mip);
  ObsModel dmt;
  ProtInter dit(g_numProt);
  outFile << "****** RunEM without DDI ..." << endl;
  i = RunEM(pm, pi, mip, micp, om, dmt, dit, outFile);
  outFile << "*** Total # EM round and joint prob:\t" << i << '\t' << LogJointProb(pm, pi, mip, micp, om, dmt, dit, outFile) << endl;
  outFile << "****** Potential without conditioning ******" << endl;
  mip.Print(outFile);
  outFile << "****** PPI observatoin model ******" << endl;
  om.Print(outFile);

  g_curjProbChange = 1;
  // micp == mip so learning w/o DDI, which converged already so only
  // collecting sufficient statistics 1st & then learn initial model
  outFile << "****** initializing DDI model ..." << endl;
  EMStep(pm, mip, micp, pi, om, dm, di);
  outFile << "*** Joint prob:\t" << LogJointProb(pm, pi, mip, micp, om, dm, di, outFile) << endl;
  outFile << "****** Potential without conditioning ******" << endl;
  mip.Print(outFile);
  outFile << "****** Potential conditioned on DDI data ******" << endl;
  micp.Print(outFile);
  outFile << "****** PPI observatoin model ******" << endl;
  om.Print(outFile);
  outFile << "****** DDI observatoin model ******" << endl;
  dm.Print(outFile);

  outFile << "****** RunEM with DDI ..." << endl;
  i = RunEM(pm, pi, mip, micp, om, dm, di, outFile);
  outFile << "*** Total # EM round and joint prob:\t" << i << '\t' << LogJointProb(pm, pi, mip, micp, om, dm, di, outFile) << endl;
  outFile << "****** Potential without conditioning ******" << endl;
  mip.Print(outFile);
  outFile << "***** Potential conditioned on DDi data *****" << endl;
  micp.Print(outFile);
  outFile << "****** PPI observatoin model ******" << endl;
  om.Print(outFile);
  outFile << "****** DDI observatoin model ******" << endl;
  dm.Print(outFile);

  // output interaction probability prediction
  Mylib::OutFopen(predictFile, argv[7]);
  Mylib::InFopen(predpairFile, argv[8]);
  Mylib::InFopen(predbindFile, argv[9]);
  PredictProtInter(predpairFile, predictFile, pm, pi, mip, micp, om);
  predpairFile.close();
  if (perType != 2) {
    PredictBinding(predbindFile, predictFile, pm, mip, micp, pi, om);
  }
  predictFile << "*** End of file." << endl;
  predictFile.close();

  // IMPROVETHRESH /= 5.0;
  NUMITERCHECK = 1;
  jprob = LogJointProb(pm, pi, mip, micp, om, dm, di, outFile);
  outFile << "Joint prob before perturbation:\t" << jprob << endl;
  switch (perType) {
  case 1:
    Mylib::InFopen(predbindFile, argv[9]);
    PertPair(pm, pi, mip, micp, om, dm, di, outFile, predbindFile, jprob, pertSize);
    predbindFile.close();
    break;
  case 2:
    Mylib::InFopen(predbindFile, argv[9]);
    PerTypes(pm, pi, mip, micp, om, dm, di, outFile, predbindFile, jprob, pertSize);
    predbindFile.close();
    break;
  case 3:
    Mylib::InFopen(predbindFile, argv[9]);
    PertMot2(pm, pi, mip, micp, om, dm, di, outFile, predbindFile, jprob, pertSize);
    predbindFile.close();
    break;
  default:
    assert(0);
  }
  outFile << "*** End of file." << endl;
  outFile.close();
  nullFile.close();
  
  return 0;
}

int RunEM(const ProtMot & pm, const ProtInter & pi, MotInterPot & mip, MotInterPot & micp, ObsModel & om, ObsModel & dm, const ProtInter & di, ofstream & outFile)
{
  int j = 0;
  double EMjProb, prevEMjProb;
  
  prevEMjProb = LogJointProb(pm, pi, mip, micp, om, dm, di, outFile);
  outFile << "*** Start of EM, before round 0. Log joint:\t" << prevEMjProb << endl;
  /*
  for (; j < 4; j++) {
    EMStep(pm, mip, micp, pi, om, dm, di);
  }
  prevEMjProb = LogJointProb(pm, pi, mip, micp, om, dm, di, outFile);
  outFile << "*** (After EM round, Log joint):\t" << j << '\t' << prevEMjProb << endl;
  */
  for (; 1; j++) {
    EMStep(pm, mip, micp, pi, om, dm, di);
    /*
    cout << "############ EMStep() debugging start: " << j << endl;
    mip.Print(cout);
    micp.Print(cout);
    om.Print(cout);
    dm.Print(cout);
    cout << LogJointProb(pm, pi, mip, micp, om, dm, di, cout) << endl;
    cout << "############ EMStep() debugging end" << endl;
    */
    if (j % NUMITERCHECK == NUMITERCHECK - 1) {
      // ??? Stop condition is simply joint probability increase less than certain
      // factor in certain # iterations. Talk to Daphne or Eran for some better way
      EMjProb = LogJointProb(pm, pi, mip, micp, om, dm, di, outFile);
      outFile << "*** (After EM round, Log joint):\t" << j << '\t' << EMjProb << endl;
      if (EMjProb < IMPROVETHRESH + prevEMjProb) {
	break;
      }
      prevEMjProb = EMjProb;
    }
  }
  return (j + 1);
}

void EMStep(const ProtMot & pm, MotInterPot & mip, MotInterPot & micp, const ProtInter & pi, ObsModel & om, ObsModel & dm, const ProtInter & di)
{
  EnM1Step(pm, mip, micp, pi, om, dm, di);
  om.M2Step();
  dm.M2Step(&micp, &mip, &di);
}

void EnM1Step(const ProtMot & pm, MotInterPot & mip, const MotInterPot & micp, const ProtInter & pi, const ObsModel & om, const ObsModel & dm, const ProtInter & di)
// E-step: soft assignment to T.B, T.S (leak varialbes), and T.I
// 1st M-step: max the noise (motif interact strength) parameter \theta_ab and leak probability \theta_s
// mip updated. Observation models of PPI and DDI accumulates sufficient statistics
// Everything else should remain unchanged
{
  int x, y, c, leakc = 0, prot1, prot2, withnb;
  double invp, leaksum = 0, s, p0o, p1o, prob0;
  const map<int, double> & mp = micp.GetActPair(); // mp: motif pair
  const map<int, INTERDATA> & ipp = pi.GetAllInter(); // ipp: interact protein pair
  map<int, INTERDATA>::const_iterator ipi, idata; // ipp iterator
  map<int, int>::const_iterator xi, yi;
  map<int, double>::const_iterator mpi;
  map<int, int> motCnt;
  map<int, double> sum;

  assert(om.IsStatEmpty() == true);
  assert(dm.IsStatEmpty() == true);
  for (ipi = ipp.begin(); ipi != ipp.end(); ++ipi) {
    prot1 = pi.RevIndex(ipi->first).first;
    prot2 = pi.RevIndex(ipi->first).second;
    const map<int, int> & xam = pm.GetMots(prot1);
    const map<int, int> & yam = pm.GetMots(prot2);
    p0o = om.GetProb(0, ipi->second); // Prob of the observation given non-interaction
    p1o = om.GetProb(1, ipi->second); // Prob of the observation given interaction
    prob0 = AlNonInterProb(pm, micp, prot1, prot2); // Prob of non-interaction given nothing
    invp = p1o / (p0o * prob0 + p1o * (1 - prob0));
    om.AddStat((1 - prob0) * invp, ipi->second); // (1 - prob0) * invp is the prob of interaction given observation
    leaksum += (pm.NumActLeak(prot1, prot2) * invp);
    leakc += pm.NumLeak(prot1, prot2);
    withnb = pm.WithNonBind(prot1, prot2);
    for (xi = xam.begin(); xi != xam.end(); ++xi) {
      for (yi = yam.begin(); yi != yam.end(); ++yi) {
	if (micp.IsAct(xi->first, yi->first) == 1) {
	  idata = di.GetInter(xi->first, yi->first);
	  motCnt[micp.Index(xi->first, yi->first)]++;
	  // use withnb to speed up
	  if (withnb == 0 || pm.IsNonBind(prot1, prot2, xi->first, yi->first) == 0) {
	    sum[micp.Index(xi->first, yi->first)] += invp;
	    // mot pair (x, y) may not have any evidence
	    if (idata != di.GetInterEnd()) {
	      dm.AddStat(micp.GetPot(xi->first, yi->first) * invp, idata->second);
	    }
	  } else { // active but forced non-binding
	    // mot pair (x, y) may not have any evidence
	    if (idata != di.GetInterEnd()) {
	      dm.AddStat(0, idata->second);
	    }
	  }
#ifdef _INACTIVE_MOTPAIR_IN_SS
	} else { // inactive motif pair
	  // If add those inactive, have to input data for those pair
	  // too. If don't add, we can remove them from input data
	  idata = di.GetInter(xi->first, yi->first);
	  // mot pair (x, y) may not have any evidence
	  if (idata != di.GetInterEnd()) {
	    dm.AddStat(0, idata->second);
	  }
#endif // _INACTIVE_MOTPAIR_IN_SS
	}
      }
    }
  }
  for (mpi = mp.begin(); mpi != mp.end(); mpi++) {
    x = micp.RevIndex(mpi->first).first;
    y = micp.RevIndex(mpi->first).second;
    s = sum[micp.Index(x, y)];
    // if the key doesn't exist, it will automatically be created to be 0
    c = motCnt[micp.Index(x, y)];
    mip.SetPot(x, y, (micp.GetPot(x, y) * s + g_aPos) / (c + g_aAll));
  }
  mip.SetLeak((micp.GetLeak() * leaksum + g_aPos) / (leakc + g_aAll));
  g_curjProbChange = 1;
}

double AlNonInterProb(const ProtMot & pm, const MotInterPot & micp, const int prot1, const int prot2)
// pm: proteins and their motif activities
// return probability that there is no actual interaction (NOT observation) between x and y (given the forced non-binding status)
// Do not return probability of actual interaction due to loss of accuracy by doing 1 - p0
{
  double p0 = pow((1 - micp.GetLeak()), pm.NumActLeak(prot1, prot2));
  map<int, int>::const_iterator xi, yi;

  const map<int, int> & xam = pm.GetMots(prot1);
  const map<int, int> & yam = pm.GetMots(prot2);
  for (xi = xam.begin(); xi != xam.end(); ++xi) {
    for (yi = yam.begin(); yi != yam.end(); ++yi) {
      if (pm.IsNonBind(prot1, prot2, xi->first, yi->first) == 0 && micp.IsAct(xi->first, yi->first) == 1) {
	p0 *= (1 - micp.GetPot(xi->first, yi->first));
      }
    }
  }
  return p0;
}

double LogJointProb(const ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ostream & outFile, const int part)
// return the joint probability of the observations with all parameters set
{
  double jLogProb = 0.0, prob0, prob1, p0o, p1o, lp = mip.GetLeak();
  int prot1, prot2, withnb;
  const map<int, INTERDATA> & ipp = pi.GetAllInter();
  map<int, INTERDATA>::const_iterator ipi, idata; // ipp iterator
  map<int, int>::const_iterator xi, yi;
  const map<int, double> & mp = mip.GetActPair(); // mp: motif pair
  map<int, double>::const_iterator mpi;

  if (part == 0 && g_curjProbChange == 0 && om.Dirty() == false && dm.Dirty() == false) {
    return g_curjProb;
  }
  outFile << "Recompute joint probability ..." << endl;

  for (ipi = ipp.begin(); ipi != ipp.end(); ++ipi) {
    prot1 = pi.RevIndex(ipi->first).first;
    prot2 = pi.RevIndex(ipi->first).second;
    if (part == 1 && IsActPos(ipi->second) == 0) {
      continue;
    }
    if (part == 2 && IsActNeg(ipi->second) == 0) {
      continue;
    }
    p0o = om.GetProb(0, ipi->second); // prob given non-interaction
    p1o = om.GetProb(1, ipi->second); // prob given interaction
    prob0 = AlNonInterProb(pm, micp, prot1, prot2);
    jLogProb += log(p0o * prob0 + p1o * (1 - prob0));

    withnb = pm.WithNonBind(prot1, prot2);
    if (withnb == 1) {
      jLogProb += pm.NumNBLeak(prot1, prot2) * log(1 - lp);
    }
    // the part below computes P(DDI) while above computes P(PPI | DDI)
    // So it's different from E-step where we always condition on DDI
    // Thus we need to use mip instead micp in the part below
    const map<int, int> & xam = pm.GetMots(prot1);
    const map<int, int> & yam = pm.GetMots(prot2);
    for (xi = xam.begin(); xi != xam.end(); ++xi) {
      for (yi = yam.begin(); yi != yam.end(); ++yi) {
	if (mip.IsAct(xi->first, yi->first) == 1) {
	  idata = di.GetInter(xi->first, yi->first);
	  // use withnb to speed up
	  if (withnb == 0 || pm.IsNonBind(prot1, prot2, xi->first, yi->first) == 0) {
	    if (idata != di.GetInterEnd()) {
	      p0o = dm.GetProb(0, idata->second);
	      p1o = dm.GetProb(1, idata->second);
	      // use mip, instead of micp !
	      prob1 = mip.GetPot(xi->first, yi->first);
	      jLogProb += log(p0o * (1 - prob1) + p1o * prob1);
	    }
	  } else { // active but forced non-binding
	    if (idata != di.GetInterEnd()) {
	      jLogProb += log(dm.GetProb(0, idata->second));
	    }
	    jLogProb += log(1 - mip.GetPot(xi->first, yi->first));
	  }
#ifdef _INACTIVE_MOTPAIR_IN_SS
	} else { // inactive motif pair
	  // If add those inactive, have to input data for those pair
	  // too. If don't add, we can remove them from input data
	  idata = di.GetInter(xi->first, yi->first);
	  if (idata != di.GetInterEnd()) {
	      jLogProb += log(dm.GetProb(0, idata->second));
	  }
#endif // _INACTIVE_MOTPAIR_IN_SS
	}
      }
    }
  }
  if (part == 0) {
    // Contribution from the Dirichlet prior. Only for part == 0 b/c
    // Riley's paper doesn't have Dirichlet prior in E score, but it
    // does use it when doing EM
    for (mpi = mp.begin(); mpi != mp.end(); mpi++) {
      jLogProb += (g_aNeg * log(1 - mpi->second) + g_aPos * log(mpi->second));
    }
    jLogProb += g_aNeg * log(1 - lp) + g_aPos * log(lp);
    g_curjProb = jLogProb;
    g_curjProbChange = 0;
    om.SetClean();
    dm.SetClean();
  }
  return jLogProb;
}

void PredictProtInter(ifstream & predpairFile, ofstream & predictFile, const ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om)
{
  int i, j, ind;
  double p0o, p1o, prob0;
  ProtInter pp(g_numProt); // for pairs to predict

  predictFile << "***s Predicted non-interaction probability for protein pairs:\t(1st, 2nd, prob w/o evidence, prob w/ evidence/H)" << endl;
  while (predpairFile >> i >> j) {
    ind = pi.Index(i, j);
    // this is just to make sure i and j are in the consistent order
    i = pi.RevIndex(ind).first;
    j = pi.RevIndex(ind).second;
    prob0 = AlNonInterProb(pm, micp, i, j); // Prob of non-interaction given nothing
    predictFile << i << '\t' << j << '\t' << AlNonInterProb(pm, mip, i, j) << ',' << prob0 << '\t';
    if (pi.IsInter(i, j) == true) { // when there is evidence
      const INTERDATA & inter = pi.GetInter(i, j)->second;
      p0o = om.GetProb(0, inter); // Prob of the observation given non-interaction
      p1o = om.GetProb(1, inter); // Prob of the observation given interaction
      predictFile << prob0 * p0o / (p0o * prob0 + p1o * (1 - prob0));
    } else {
      predictFile << "H";
    }
    predictFile << endl;
  }
  predictFile << "***e Predicted non-interaction probability for protein pairs." << endl;
}

void PredictBinding(ifstream & predbindFile, ofstream & predictFile, const ProtMot & pm, const MotInterPot & mip, const MotInterPot & micp, const ProtInter & pi, const ObsModel & om)
{
  set<int>::const_iterator ipi; // ipp iterator
  map<int, int>::const_iterator xi, yi;
  int i, j, ind;
  double p0o, p1o, prob0, invp;
  ProtInter pp(g_numProt);

  predictFile << "***s Binding variable:\t(1st prot, 2nd prot, 1st mot, 2nd mot, mot affinity, P(B=1) w/ evidence/H)" << endl;
  while (predbindFile >> i >> j) {
    ind = pi.Index(i, j);
    // this is just to make sure i and j are in the consistent order
    i = pi.RevIndex(ind).first;
    j = pi.RevIndex(ind).second;
    const map<int, int> & xam = pm.GetMots(i);
    const map<int, int> & yam = pm.GetMots(j);
    if (pi.IsInter(i, j) == true) {
      const INTERDATA & inter = pi.GetInter(i, j)->second;
      p0o = om.GetProb(0, inter); // Prob of the observation given non-interaction
      p1o = om.GetProb(1, inter); // Prob of the observation given interaction
      prob0 = AlNonInterProb(pm, micp, i, j); // Prob of non-interaction given nothing
      invp = p1o / (p0o * prob0 + p1o * (1 - prob0));
      for (xi = xam.begin(); xi != xam.end(); ++xi) {
	for (yi = yam.begin(); yi != yam.end(); ++yi) {
	  if (pm.IsNonBind(i, j, xi->first, yi->first) == 0 && micp.IsAct(xi->first, yi->first) == 1) {
	    predictFile << i << '\t' << j << '\t' << xi->first << '\t' << yi->first << '\t' << mip.GetPot(xi->first, yi->first) << ',' << micp.GetPot(xi->first, yi->first) << '\t' << micp.GetPot(xi->first, yi->first) * invp << endl;
	  }
	}
      }
    } else {
      for (xi = xam.begin(); xi != xam.end(); ++xi) {
	for (yi = yam.begin(); yi != yam.end(); ++yi) {
	  if (pm.IsNonBind(i, j, xi->first, yi->first) == 0 && micp.IsAct(xi->first, yi->first) == 1) {
	    predictFile << i << '\t' << j << '\t' << xi->first << '\t' << yi->first << '\t' << mip.GetPot(xi->first, yi->first) << ',' << micp.GetPot(xi->first, yi->first) << "\tH" << endl;
	  }
	}
      }
    }
  }
  predictFile << "***e Binding variable." << endl;
}

void PertPair(ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ofstream & outFile, ifstream & predbindFile, double jprob, int pertSize)
{
  int i, j, k, t, ind;
  set<int>::const_iterator ipi; // pp iterator
  map<int, int>::const_iterator xi, yi;
  ProtInter pp(g_numProt);
  ofstream nullFile("/dev/null");

  while (predbindFile >> i >> j) {
    // predict only for interacting pairs
    if (pi.IsInter(i, j) == true) {
      // this is just to make sure i and j are in the consistent order
      ind = pi.Index(i, j);
      for (t = 0; t < 2; t++) {
	if (t == 0) {
	  i = pi.RevIndex(ind).first;
	  j = pi.RevIndex(ind).second;
	} else {
	  i = pi.RevIndex(ind).second;
	  j = pi.RevIndex(ind).first;
	}
	const map<int, int> & xam = pm.GetMots(i);
	for (xi = xam.begin(); xi != xam.end(); ++xi) {
	  if (pertSize == -1 || xi->first < pertSize) {
	    pm.SetNonBind(i, j, xi->first);
	    outFile << "*** SetNonBind (prot1, mot1, prob2, numLeak, numNBLeak, numActLeak):\t" << i << '\t' << xi->first << '\t' << j << '\t' << pm.NumLeak(i, j) << '\t' << pm.NumNBLeak(i, j) << '\t' << pm.NumActLeak(i, j) << endl;
	    MotInterPot mipt(mip);
	    ObsModel omt(om);
	    MotInterPot micpt(micp);
	    ObsModel dmt(dm);
	    // need to set this b/c NonBind in pm updated & switching
	    // from mip, om, micp, and dm to mipt, omt, micpt, and dmt
	    g_curjProbChange = 1;
	    om.SetDirty();
	    omt.SetDirty();
	    dm.SetDirty();
	    dmt.SetDirty();
	    outFile << "*** Non-interaction probability after forcing an active motif and a protein to be non-binding (Prot1, Mot1, Prot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << AlNonInterProb(pm, micpt, i, j) << endl;
	    outFile << "*** Likelihood change after forcing an active motif and a protein to be non-binding (Prot1, Mot1, Prot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile) - jprob << endl;
	    k = RunEM(pm, pi, mipt, micpt, omt, dmt, di, nullFile);
	    outFile << "*** Total # EM round (Prot1, Mot1, Prot2, EM):\t" << i << '\t' << xi->first << '\t' << j << '\t' << k << endl;
	    outFile << "*** Non-interaction probability after reconverge (Prot1, Mot1, Prot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << AlNonInterProb(pm, micpt, i, j) << endl;
	    outFile << "*** Likelihood change after reconverge (Prot1, Mot1, Prot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile) - jprob << endl;
	    // need to set this b/c switching from mipt, omt, micpt, and
	    // dmt to mip, om, micp, and dm as well as NonBind of pm
	    g_curjProbChange = 1;
	  }
	}
      }
    }
  }
  pm.UnsetNonBind();
  g_curjProbChange = 1;
}

void PertMot2(ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ofstream & outFile, ifstream & predbindFile, double jprob, int pertSize)
{
  int i, j, k;
  set<int>::const_iterator ipi; // pp iterator
  map<int, int>::const_iterator xi, yi;
  ProtInter pp(g_numProt);
  ofstream nullFile("/dev/null");

  while (predbindFile >> i >> j) {
    // predict only for interacting pairs
    if (pi.IsInter(i, j) == true) {
      const map<int, int> & xam = pm.GetMots(i);
      const map<int, int> & yam = pm.GetMots(j);
      for (xi = xam.begin(); xi != xam.end(); ++xi) {
	for (yi = yam.begin(); yi != yam.end(); ++yi) {
	  if (pertSize == -1 || xi->first < pertSize && yi->first < pertSize) {
	    pm.SetNonBind(i, j, xi->first, yi->first);
	    MotInterPot mipt(mip);
	    ObsModel omt(om);
	    MotInterPot micpt(micp);
	    ObsModel dmt(dm);
	    // need to set this b/c NonBind in pm updated & switching
	    // from mip, om, micp, and dm to mipt, omt, micpt, and dmt
	    g_curjProbChange = 1;
	    om.SetDirty();
	    omt.SetDirty();
	    dm.SetDirty();
	    dmt.SetDirty();
	    outFile << "*** Non-interaction probability after forcing an active motif and a protein to be non-binding (Prot1, Mot1, Prot2, Mot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << yi->first << '\t' << AlNonInterProb(pm, micpt, i, j) << endl;
	    outFile << "*** Likelihood change after forcing an active motif and a protein to be non-binding (Prot1, Mot1, Prot2, Mot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << yi->first << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile) - jprob << endl;
	    k = RunEM(pm, pi, mipt, micpt, omt, dmt, di, nullFile);
	    outFile << "*** Total # EM round (Prot1, Mot1, Prot2, Mot2, EM):\t" << i << '\t' << xi->first << '\t' << j << '\t' << yi->first << '\t' << k << endl;
	    outFile << "*** Non-interaction probability after reconverge (Prot1, Mot1, Prot2, Mot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << yi->first << '\t' << AlNonInterProb(pm, micpt, i, j) << endl;
	    outFile << "*** Likelihood change after reconverge (Prot1, Mot1, Prot2, Mot2, Prob):\t" << i << '\t' << xi->first << '\t' << j << '\t' << yi->first << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile) - jprob << endl;
	    // need to set this b/c switching from mipt, omt, micpt, and
	    // dmt to mip, om, micp, and dm as well as NonBind of pm
	    g_curjProbChange = 1;
	  }
	}
      }
    }
  }
  pm.UnsetNonBind();
  g_curjProbChange = 1;
}

void PerTypes(ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ofstream & outFile, ifstream & predbindFile, double jprob, int pertSize)
{
  int ind, x, y, k;
  // const map<int, double> & mp = micp.GetActPair(); // mp: motif pair
  // map<int, double>::const_iterator mpi;
  ofstream nullFile("/dev/null");

  while (predbindFile >> x >> y) {
    if (pertSize == -1 || x < pertSize && y < pertSize) {
      MotInterPot mipt(mip);
      ObsModel omt(om);
      MotInterPot micpt(micp);
      ObsModel dmt(dm);
      // need to set this b/c NonBind in pm updated & switching
      // from mip, om, micp, and dm to mipt, omt, micpt, and dmt
      g_curjProbChange = 1;
      om.SetDirty();
      omt.SetDirty();
      dm.SetDirty();
      dmt.SetDirty();
      // ind = mpi->first;
      ind = micp.Index(x, y);
      mipt.DelPotIndex(ind);
      micpt.DelPotIndex(ind);
      // x = micp.RevIndex(ind).first;
      // y = micp.RevIndex(ind).second;
      outFile << "*** Likelihood change after forcing a pair of motif types to be non-binding (Mot1, Mot2, Prob):\t" << x << '\t' << y << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile) - jprob << endl;
      outFile << "*** Positive likelihood after forcing a pair of motif types to be non-binding (Mot1, Mot2, Prob):\t" << x << '\t' << y << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile, 1) << endl;
      k = RunEM(pm, pi, mipt, micpt, omt, dmt, di, nullFile);
      outFile << "*** Total # EM round (Mot1, Mot2, EM):\t" << x << '\t' << y << '\t' << k << endl;
      outFile << "*** Likelihood change after reconverge (Mot1, Mot2, Prob):\t" << x << '\t' << y << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile) - jprob << endl;
      outFile << "*** Positive likelihood after reconverge (Mot1, Mot2, Prob):\t" << x << '\t' << y << '\t' << LogJointProb(pm, pi, mipt, micpt, omt, dmt, di, outFile, 1) << endl;
      // need to set this b/c switching from mipt, omt, micpt, and
      // dmt to mip, om, micp, and dm as well as NonBind of pm
      g_curjProbChange = 1;
    }
  }
  g_curjProbChange = 1;
}

// Read in and set all data
void ReadFiles(ProtMot & pm, ProtInter & pi, MotInterPot & mip, ObsModel & om, ObsModel & dm, ProtInter & di, ifstream & protLenFile, ifstream & motCoverFile, ifstream & pmFile, ifstream & protFile, ifstream & modelFile, ifstream & motFile, ofstream & outFile, const int leakGroup, ifstream & ddiFile, ifstream & ddmdlFile, ifstream & ddioptFile)
{
  int i, x, y, z, cnt;
  double val;

  pm.SetLeakGroup(leakGroup);
  // load protein length file to pm
  while (protLenFile >> x >> y) {
    assert(x >= 0);
    assert(x < g_numProt);
    assert(y > 0);
    assert(pm.GetProtLen(x) == -1);
    pm.AddProtLen(x, y);
  }
  for (i = 0; i < g_numProt; i++) {
    assert(pm.GetProtLen(i) > 0);
  }
  // load motif coverage file to pm
  while (motCoverFile >> x >> y) {
    assert(x >= 0);
    assert(x < g_numProt);
    assert(y >= 0);
    assert(y <= pm.GetProtLen(x));
    assert(pm.GetMotCover(x) == -1);
    pm.AddMotCover(x, y);
  }
  for (i = 0; i < g_numProt; i++) {
    assert(pm.GetMotCover(i) >= 0);
  }
  // load protein-motif file. Set pm
  cnt = 0;
  while (pmFile >> x >> y >> z) {
    assert(x >= 0);
    assert(x < g_numProt);
    assert(y >= 0);
    assert(y < g_numMot);
    assert(pm.IsMot(x, y) == 0);
    assert(z > 0);
    pm.AddMot(x, y, z);
    cnt++;
  }
  pmFile.close();
  outFile << "*** Total # motif appearances:\t" << cnt << endl;
  ReadModelPPI(om, pi, modelFile, protFile, outFile);
  // load motif file. Set mip
  cnt = 0;
  while (motFile >> x >> y >> val) {
    assert(x >= 0);
    assert(x < g_numMot);
    assert(y >= 0);
    assert(y < g_numMot);
    assert(val > 0.0 && val < 1.0);
    assert(mip.IsAct(x, y) == 0);
    mip.AddPot(x, y, val);
    cnt++;
  }
  motFile.close();
  outFile << "*** # active motif pairs:\t" << cnt << endl;
  switch (pm.GetLeakType()) {
  case 0:
  case 2:
    mip.SetLeak(INITLEAK0PROB);
    break;
  case 1:
    mip.SetLeak(sqrt(leakGroup) * INITLEAKPERVAR);
    break;
  default:
    assert(0);
  }
  outFile << "*** Initial leak prob:\t" << mip.GetLeak() << endl;
  ReadModelPPI(dm, di, ddmdlFile, ddiFile, outFile, &ddioptFile, &mip);
}

void ReadModelPPI(ObsModel & om, ProtInter & pi, ifstream & modelFile, ifstream & protFile, ofstream & outFile, ifstream * ddioptFile, const MotInterPot * mip)
{
  int i, x, y, cnt, id[2], size;
  map<int, int> modelid;
  double val;

  // load model type file. modelid: mapping from ID we use in input to ID we store in om
  cnt = 0;
  while (modelFile >> x) {
    assert(x >= 0);
    assert(modelid.find(x) == modelid.end());
    modelid[x] = cnt++;
    y = modelid[x];
    assert(y + 1 == cnt);
    om.AddDist(ReadDistribution(modelFile));
  }
  // load protein interaction file. Set pi
  cnt = 0;
  while (protFile >> x >> y >> val >> id[0] >> id[1]) {
    assert(x >= 0);
    assert(y >= 0);
    if (ddioptFile == NULL) { // It's PPI data instead of DDI
      assert(x < g_numProt);
      assert(y < g_numProt);
      assert(x != y);
    } else { // It's DDI data
      assert(x < g_numMot);
      assert(y < g_numMot);
    }
    AddOneInter(x, y, val, id, pi, cnt, modelid, om, mip);
  }
  protFile.close();
  if (ddioptFile != NULL) {
    assert(mip != NULL); // b/c we need to use it below
    const map<int, double> & mp = mip->GetActPair();
    map<int, double>::const_iterator mpi;
    while ((*ddioptFile) >> i >> val >> id[0] >> id[1] >> size) {
      if (i == 0 || i == 2) { // equal pairs covered by default value
	for (x = 0; x < size; x++) {
	  if (pi.GetInterVal(x, x, id) == false) {
	    AddOneInter(x, x, val, id, pi, cnt, modelid, om, mip);
	  }
	}
      }
      if (i == 1 || i == 2) { // unequal pairs covered by default value
#ifdef _INACTIVE_MOTPAIR_IN_SS
	for (x = 0; x < size; x++) {
	  for (y = 0; y < x; y++) {
#else
	for (mpi = mp.begin(); mpi != mp.end(); mpi++) {
	  x = mip->RevIndex(mpi->first).first;
	  y = mip->RevIndex(mpi->first).second;
	  if (x < size && y < size && x != y) {
#endif // _INACTIVE_MOTPAIR_IN_SS
	    if (pi.GetInterVal(x, y, id) == false) {
	      AddOneInter(x, y, val, id, pi, cnt, modelid, om, mip);
	    }
	  }
	}
      }
    }
  }
  outFile << "*** # noisy observations:\t" << cnt << endl;
}

void AddOneInter(int x, int y, double val, int id[], ProtInter & pi, int & cnt, const map<int, int> & modelid, const ObsModel & om, const MotInterPot * mip)
{
  int i;
  map<int, int>::const_iterator imdl;

#ifndef _INACTIVE_MOTPAIR_IN_SS
  if (mip != NULL && mip->IsAct(x, y) == 0) {
    return;
  }
#endif // _INACTIVE_MOTPAIR_IN_SS

  for (i = 0; i < 2; i++) {
    if (id[i] >= 0) {
      imdl = modelid.find(id[i]);
      assert(imdl != modelid.end());
      id[i] = imdl->second;
      assert(om.DistAssert(id[i], val) == true);
    } else {
      assert(id[i] == -1 || id[i] == -2);
    }
  }
  pi.AddInter(x, y, val, id[0], id[1]);
  cnt++;
}

void PrintUsage(ostream & out)
{
  out << "Usage: exe" << endl;
  out << "  [-f <numprot> <nummot>]" << endl;
  out << "  [-l <leaktype>]" << endl;
  out << "  [-p <pertype>]" << endl;
  out << "  [-d <ddi file> <ddi model file> <ddi opt file>]" << endl;
  out << "  [-n <Dirichlet prior Neg / Pos>]" << endl;
  out << "  [-s <pertsize>]" << endl;
  out << "  <observed protein interaction file>" << endl;
  out << "  <model type file>" << endl;
  out << "  <protein-motif content file>" << endl;
  out << "  <active motif pair file>" << endl;
  out << "  <inactive but may activate motif pair file>" << endl;
  out << "  <waiting list motif pair file>" << endl;
  out << "  <output file>" << endl;
  out << "  <prediction file>" << endl;
  out << "  <protein pairs to predict interaction>" << endl;
  out << "  <protein pairs to predict binding variables>" << endl;
  out << "  <Dirichlet prior: Pos>" << endl;
  out << "  <protein length file> f1: prot. f2: len" << endl;
  out << "  <# a.a. pairs in leakProb> (integer)" << endl;
  out << "  <motif coverage file>" << endl;
  out << endl;
  out << "Note: The names for proteins must exactly cover [0, NUMPROT) because we "
      << "assume the pairs not listed to be non-interacting. All names for motifs "
      << "must be in [0, NUMMOT)." << endl;
  out << endl;
  out << "<numprot>:" << endl;
  out << "  # total proteins considered" << endl;
  out << "<nummot>:" << endl;
  out << "  # total motifs considered" << endl;
  out << endl;
  out << "<leaktype>:" << endl;
  out << "  0: always 1 leak variable" << endl;
  out << "  1: # leak vars proportional to the square root of non-motif a.a. pair" << endl;
  out << "  2: no leak variable like in Riley et al." << endl;
  out << "Default: 1" << endl;
  out << "<# a.a. pairs in leakProb> only meaningful for <leaktype> 1" << endl;
  out << endl;
  out << "<pertype>:" << endl;
  out << "  1: force a motif and a protein to be non-binding. Our model" << endl;
  out << "  2: force pairs between motif types to be non-binding. Riley's model" << endl;
  out << "  3: force a pair of motifs in a protein pair to be non-binding" << endl;
  out << "Default: 1" << endl;
  out << endl;
  out << "<ddi file> <ddi model file> <ddi opt file>:" << endl;
  out << "Default: no evidence under the ddi binding vars" << endl;
  out << endl;
  out << "<ddi file>: domain-domain binding noisy indicator" << endl;
  out << "f1: mot1. f2: mot2. f3: observed value. f4: ID of model "
      << "to use when actual value is 0. f5: model ID when actual "
      << "value is 1." << endl;
  out << "Mot pairs must be all different. f1 and f2 can be the same. "
      << "Pairs of the same model ID share same parameters & learned "
      << "together. ID -1 means the same as actual val, ie. f3 = 1 "
      << "given actual val = 1 & 0 given actual val = 0. Note that f4 "
      << "can be -1 while f5 can be other value, or vice versa, which "
      << "means f3 is always 0 given actual val = 0, while f3 can be "
      << "other dist given actual val = 1. ID -2 means uniform dist "
      << "over [0, 1], ie. f3 = any value in [0, 1] w/ prob 1. All "
      << "other IDs has to be >= 0." << endl;
  out << "Warning: if the same observation appear twice, they will "
      << "be considered to be two independent observations and "
      << "strengthen the belief." << endl;
  out << endl;
  out << "<ddi model file>: DDI observation model" << endl;
  out << "f1: model ID in <ddi file>. f2: model type. Rest fields: "
      << "additional parameter for the model type. See <model type "
      << "file> below for description." << endl;
  out << "f1 is >= 0 & should cover all IDs appeared in <ddi file> "
      << "except for ID -1 and -2, which has special meaning. No ID "
      << "should appear twice." << endl;
  out << endl;
  out << "<ddi opt file>: additional options for <ddi file>" << endl;
  out << "f1: option:" << endl;
  out << " 0: all pair of same mots should be included." << endl;
  out << " 1: all pair of different mots should be included." << endl;
  out << " 2: all pair of mots should be included." << endl;
  out << "f2: default val. f3: model1. f4: model2. Only add the pair "
      << "with default value if it is not in <ddi file> with the same "
      << "(model1, model2) id[]. f5: # mots. Only add pair based on "
      << "when both mots have index in [0, f5)." << endl;
  out << endl;
  out << "<Dirichlet prior Neg / Pos>:" << endl;
  out << "See <Dirichlet prior: Pos> below" << endl;
  out << endl;
  out << "<pertsize>:" << endl;
  out << "Perturb only if mots have index in [0, <pertsize>)" << endl;
  out << endl;
  out << "<observed protein interaction file>:" << endl;
  out << "f1: 1st prot. f2: 2nd prot. f3: observed value. f4: ID of "
      << "model to use when actual value is 0. f5: model ID when "
      << "actual value is 1." << endl;
  out << "Protein pairs must be all different. f1 and f2 must be "
      << "different. Pairs of the same model ID share same parameters "
      << "and learned together. ID -1 means the same as actual val, "
      << "ie. f3 = 1 given actual val = 1 & 0 given actual val = 0. "
      << "Note that f4 can be -1 while f5 can be other value (or vice "
      << "versa), which means f3 is always 0 given actual val = 0, "
      << "while f3 can be other dist given actual val = 1. ID -2 "
      << "means uniform dist over [0, 1], ie. f3 = any value in "
      << "[0, 1] w/ prob 1. All other IDs has to be >= 0." << endl;
  out << "Warning: if the same observation appear twice, they will "
      << "be considered to be two independent observations and "
      << "strengthen the belief." << endl;
  out << endl;
  out << "<model type file>:" << endl;
  out << "f1: model ID in <observed protein interaction file>. f2: "
      << "model type. Rest fields: additional parameter for the model "
      << "type:" << endl;
  out << " OM_BINARY: init prob of being 0." << endl;
  out << " OM_GAUSSIAN: init mean; init variance." << endl;
  out << " OM_EXPONENTIAL: shift(c); right(0)/left(1); init l" << endl;
  out << "  l*exp(-lambda*(x-c)), x>=c if right so obsval >=c" << endl;
  out << "  l*exp(-lambda*(c-x)), x<=c if left so osbval <=c" << endl;
  out << "  shift and right/left are fixed in learning." << endl;
  out << " OM_MIXTURE: max # iterations when learning (m);" << endl;
  out << "  # distributions in the mixture (s); then a new" << endl;
  out << "  line for each distribution:" << endl;
  out << "   initial weight (w); distribution type (t); list" << endl;
  out << "   of parameters as described above" << endl;
  out << " OM_UNIFORM: begin; end; inclusiveness:" << endl;
  out << "  0: both ends open. 1: begin open, end close" << endl;
  out << "  2: begin close, end open. 3: both ends close" << endl;
  out << "  'OM_UNIFORM 0 1 3' is the same as model ID -2" << endl;
  out << "f1 is >= 0 & should cover all IDs appeared in <observed "
      << "protein interaction file> except for ID -1 and -2, which "
      << "has special meaning. No ID should appear twice." << endl;
  out << endl;
  out << "<protein-motif content file>" << endl;
  out << "f1: prot. f2: mot. f3: len of the mot on the prot." << endl;
  out << "Protein-motif pairs must be all different. i.e. each (prot, mot) pair "
      << "should appear only once. If they have multiple copies of the same motif "
      << "on a protein, their lengths should be summed up." << endl;
  out << endl;
  out << "{active,inactive but may activate,waiting list} motif pair file>:" << endl;
  out << "f1: 1st mot. f2: 2nd mot." << endl;
  out << "Motif pairs in must be all different. f1 and f2 may be the same. Motif "
      << "pairs in <waiting list> should be ranked from high to low based on the "
      << "priority it being take into consideration." << endl;
  out << endl;
  out << "<output file>:" << endl;
  out << "Log in a easy to process format." << endl;
  out << endl;
  out << "<prediction file>:" << endl;
  out << "First part (Predicted interaction probability for protein pairs):" << endl;
  out << "f1: 1st prot. f2: 2nd prot. f3: non-inter prob without evidence. "
      << "f4: non-inter prob given evidence or 'H' if no evidence." << endl;
  out << "Second part (prediction on binding var):" << endl;
  out << "f1: 1st prot. f2: 2nd prot. f3: 1st mot. f4: 2nd mot. f5: mot affinity. "
      << "f6: binding prob given evidence or 'H' if no evidence." << endl;
  out << "Go over all mot pairs on protein pairs in "
      << "<protein pairs to predict binding variables>." << endl;
  out << endl;
  out << "<protein pairs to predict interaction>:" << endl;
  out << "f1: 1st prot. f2: 2nd prot." << endl;
  out << endl;
  out << "<protein pairs to predict binding variables>:" << endl;
  out << "f1: 1st prot. f2: 2nd prot." << endl;
  out << endl;
  out << "<Dirichlet prior: Pos>:" << endl;
  out << "<Dirichlet prior: Pos> = g_aPos" << endl;
  out << "<Dirichlet prior Neg / Pos> = g_aPos / g_aNeg" << endl;
  out << "Prior over motif affinity:" << endl;
  out << "\theta_pos^g_aPos \theta_neg^g_aNeg / N, which is" << endl;
  out << "D(g_aPos + 1, g_aNeg + 1). We don't use g_aPos + 1" << endl;
  out << "as parameter b/c when we do MAP estimate in M-step," << endl;
  out << "we use g_aPos and g_aNeg as additional counts." << endl;
  out << endl;
  out << "<protein length file>:" << endl;
  out << "f1: prot. f2: len." << endl;
  out << endl;
  out << "<# a.a. pairs in leakProb>:" << endl;
  out << "refers to how many non-motif a.a. pairs between two proteins do we want "
      << "to group them into one leakProb variable. If it's larger, we will have "
      << "less leakProb variables and computation would be faster and EM more "
      << "accurate. However, the bio modeling has lower resolution." << endl;
  out << endl;
  out << "<motif coverage file>:" << endl;
  out << "f1: prot. f2: motif coverage." << endl;
  out << "Total a.a. coverage for each protein by all motifs in the training set. It "
      << "may not equal the sum of all motif lengths in <protein-motif content file> "
      << "because of overlap." << endl;
}
