#ifndef _MAIN_H
#define _MAIN_H

#include "first.h"
#include "varParaMem.h"
typedef vector<int> intvec;

void ReadOption(int & argc, char ** & argv, int & leakType, int & perType, ofstream & ddiFile, ofstream & ddmdlFile, ofstream & ddioptFile);
int RunEM(const ProtMot & pm, const ProtInter & pi, MotInterPot & mip, MotInterPot & micp, ObsModel & om, ObsModel & dm, const ProtInter & di, ofstream & outFile);
void EMStep(const ProtMot & pm, MotInterPot & mip, MotInterPot & micp, const ProtInter & pi, ObsModel & om, ObsModel & dm, const ProtInter & di);
void EnM1Step(const ProtMot & pm, MotInterPot & mip, const MotInterPot & micp, const ProtInter & pi, const ObsModel & om, const ObsModel & dm, const ProtInter & di);
double AlNonInterProb(const ProtMot & pm, const MotInterPot & micp, const int prot1, const int prot2);
// Log likelihood. Part:
//   0: consider all protein pairs
//   1: consider only pairs w/ actual positive observation (IsActPos() == 1)
//   2: consider only pairs w/ actual negative observation (IsActNeg() == 1)
double LogJointProb(const ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ostream & outFile, const int part = 0);
void PredictProtInter(ifstream & predpairFile, ofstream & predictFile, const ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om);
void PredictBinding(ifstream & predbindFile, ofstream & predictFile, const ProtMot & pm, const MotInterPot & mip, const MotInterPot & micp, const ProtInter & pi, const ObsModel & om);
void PertPair(ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ofstream & outFile, ifstream & predbindFile, double jprob = 0.0, int pertSize = -1);
// force a pair of motifs in a protein pair to be non-binding
void PertMot2(ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ofstream & outFile, ifstream & predbindFile, double jprob = 0.0, int pertSize = -1);
// Force a pair of motif types to be non-binding just like in Riley's model
// predbindFile: pairs of motif types we need to pertube
void PerTypes(ProtMot & pm, const ProtInter & pi, const MotInterPot & mip, const MotInterPot & micp, const ObsModel & om, const ObsModel & dm, const ProtInter & di, ofstream & outFile, ifstream & predbindFile, double jprob = 0.0, int pertSize = -1);
// Also initialize both mip and om
void ReadFiles(ProtMot & pm, ProtInter & pi, MotInterPot & mip, ObsModel & om, ObsModel & dm, ProtInter & di, ifstream & protLenFile, ifstream & motCoverFile, ifstream & pmFile, ifstream & protFile, ifstream & modelFile, ifstream & motFile, ofstream & outFile, const int leakGroup, ifstream & ddiFile, ifstream & ddmdlFile, ifstream & ddioptFile);
void ReadModelPPI(ObsModel & om, ProtInter & pi, ifstream & modelFile, ifstream & protFile, ofstream & outFile, ifstream * ddioptFile = NULL, const MotInterPot * mip = NULL);
// When mip non-NULL, add only if (x, y) is active mot pair
void AddOneInter(int x, int y, double val, int id[], ProtInter & pi, int & cnt, const map<int, int> & modelid, const ObsModel & om, const MotInterPot * mip);
void PrintUsage(ostream & out);

#endif // _MAIN_H
